'use strict';

/**
 * @param {Egg.Application} app - egg application
 */
module.exports = app => {
  const { router, controller } = app;

  // 开放接口 （get请求为开放接口）
  router.post('/api/power/login', controller.login.adminlogin); // 管理员帐号密码登录
  router.get('/api/qrcodeToken', controller.weixin.qrcodeToken); // uuid 换 token
  router.get('/api/getuuid', controller.weixin.getuuid); // 获取uuid
  router.get('/api/qrcodelogin/:uuid', controller.weixin.qrcodelogin); // 二维码登录
  router.get('/api/check', controller.weixin.check); // 查询二维码状态
  router.get('/api/weixin/getopenid', controller.weixin.auth); // 获取openid token
  router.get('/api/mqtt/:key', controller.weixin.mqtt); // 发送mq消息

  // 以下接口不开放
  router.post('/api/weixin/orderPay', controller.weixin.orderPay); // 预支付交易单
  router.get('/api/weixin/pay', controller.weixin.pay); // 发起支付接口
  router.post('/api/weixin/orderClose', controller.weixin.orderClose); // 关闭订单
  router.post('/api/weixin/pushMould', controller.weixin.pushMould); // 模板下发
  router.get('/api/weixin/bind', controller.weixin.bind); // 管理员绑定微信

  // 日志接口
  // router.post('/api/log/create', controller.log.create);
  router.delete('/api/log/delete/:_id', controller.log.delete);
  router.get('/api/log/query', controller.log.query);

  // 管理员接口
  router.post('/api/admin/create', controller.admin.create);
  router.post('/api/admin/update', controller.admin.update);
  router.delete('/api/admin/delete/:_id', controller.admin.delete);
  router.get('/api/admin/query', controller.admin.query);
  router.get('/api/admin/fetch/:_id', controller.admin.fetch);
  router.post('/api/editPwa', controller.login.editPwa);

  // 用户接口 开放
  router.post('/api/user/create', controller.user.create);
  router.post('/api/user/update', controller.user.update);
  router.delete('/api/user/delete/:_id', controller.user.delete);
  router.get('/api/user/query', controller.user.query);
  router.get('/api/user/fetch/:_id', controller.user.fetch);

  // 单页接口
  router.post('/api/pages/create', controller.pages.create);
  router.post('/api/pages/update', controller.pages.update);
  router.delete('/api/pages/delete/:_id', controller.pages.delete);
  router.get('/api/pages/query', controller.pages.query);
  router.get('/api/pages/fetch/:code', controller.pages.fetch);

  // 内容接口
  router.post('/api/content/create', controller.content.create);
  router.post('/api/content/update', controller.content.update);
  router.delete('/api/content/delete/:_id', controller.content.delete);
  router.get('/api/content/query', controller.content.query);
  router.get('/api/content/fetch/:_id', controller.content.fetch);

  // 医院接口
  router.post('/api/hospital/create', controller.hospital.create);
  router.post('/api/hospital/update', controller.hospital.update);
  router.delete('/api/hospital/delete/:_id', controller.hospital.delete);
  router.get('/api/hospital/query', controller.hospital.query);
  router.get('/api/hospital/fetch/:_id', controller.hospital.fetch);

  // 科室接口
  router.post('/api/subject/create', controller.subject.create);
  router.post('/api/subject/update', controller.subject.update);
  router.delete('/api/subject/delete/:_id', controller.subject.delete);
  router.get('/api/subject/query', controller.subject.query);
  router.get('/api/subject/fetch/:_id', controller.subject.fetch);

  // 专家接口
  router.post('/api/specialist/create', controller.specialist.create);
  router.post('/api/specialist/update', controller.specialist.update);
  router.delete('/api/specialist/delete/:_id', controller.specialist.delete);
  router.get('/api/specialist/query', controller.specialist.query);
  router.get('/api/specialist/fetch/:_id', controller.specialist.fetch);

  // 订单接口
  router.post('/api/order/create', controller.order.create);
  router.post('/api/order/update', controller.order.update);
  router.delete('/api/order/delete/:_id', controller.order.delete);
  router.get('/api/order/query', controller.order.query);
  router.get('/api/order/fetch/:_id', controller.order.fetch);
  router.post('/api/order/updatestatus', controller.order.updatestatus); // 修改订单状态

  // TODO: 自动配置路由,将所有以‘Action’结尾的方法自动进行路由注册
  Object.keys(app.controller).forEach(key => {
    const c = app.controller[key];
    Object.keys(c).forEach(a => {
      if (a.endsWith('Action')) {
        const p = a.substr(0, a.length - 6);
        app.all(`/${key}${p === 'index' ? '' : ('/' + p)}`, `${key}.${a}`);
      }
    });
  });
};
