'use strict';

const Service = require('egg').Service;
const assert = require('assert');
const moment = require('moment');
const routerMethod = require('../middleware/routerMethod');
const routerMondel = require('../middleware/routerMondel');
const jwt = require('jsonwebtoken');
class LogService extends Service {
  async create({ mondel, method, data, result, userName, date }) {
    assert(mondel, '模块不存在');
    assert(method, '类型不存在');
    assert(data, '数据不存在');
    assert(result, '结果不存在');
    assert(date, '时间不存在');
    const { Log: model } = this.ctx.model;
    const createAt = moment().format('x');
    try {
      await model.create({ data, method, createAt, mondel, result, userName, date });
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '添加失败' });
    }
  }
  async del({ id }) {
    assert(id, 'id不存在');
    const { Log: model } = this.ctx.model;
    try {
      await model.findById(id).remove();
      return { errmsg: '', errcode: 0 };
    } catch (error) {
      throw new Error({ errcode: -2001, errmsg: '删除失败' });
    }
  }
  async query({ skip, limit, mondel, method, result }) {
    const { Log: model } = this.ctx.model;
    const filter = {};
    if (mondel) filter.mondel = mondel;
    if (method) filter.method = method;
    if (result) filter.result = result;
    try {
      const total = await model.find({ ...filter });
      let res;
      if (skip && limit) {
        res = await model.find({ ...filter }).skip(Number(skip) * Number(limit)).limit(Number(limit));
      } else {
        res = await model.find({ ...filter });
      }
      return { errmsg: '', errcode: 0, data: res, total: total.length };
    } catch (error) {
      console.log(error);
      throw new Error({ errcode: -2001, errmsg: '查询失败' });
    }
  }
  async init() {
    const { ctx } = this;
    const { url, method, body } = ctx.request;
    let decode = {};
    const date = moment().format('YYYY-MM-DD HH:mm:ss');
    if (method !== 'GET') {
      // token
      const tokenstr = ctx.request.header.authorization;
      if (tokenstr !== 'undefined' && tokenstr) {
        const token = tokenstr.substring(7);
        decode = jwt.verify(token, this.config.jwt.secret);
      } else {
        decode = body;
      }
      const str = url.split('/');
      const methods = routerMethod[str[3]];
      const mondel = routerMondel[str[2]];
      if (methods && mondel) {
        return { mondel, method: methods, data: body, userName: decode.userName, date };
      }
      return false;
    }
  }
}

module.exports = LogService;
