'use strict';

const assert = require('assert');
const _ = require('lodash');
const { BusinessError, ErrorCode } = require('naf-core').Error;
const { NafService } = require('naf-framework-mongoose/lib/service');
const jwt = require('jsonwebtoken');

class JwtLoginService extends NafService {
  constructor(ctx) {
    super(ctx, 'naf_user_info');
    this.model = this.ctx.model.Admin;
    this.dept = this.ctx.service.dept;
    this.tag = this.ctx.service.tag;
  }

  async adminlogin({ userName, password }) {
    // TODO:参数检查和默认参数处理
    assert(userName);
    assert(password);
    // TODO:检查useridh和mobile
    const entity = await this.model.findOne({ userName });
    if (entity == null) {
      throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '用户不存在');
    }
    if (!entity.password || entity.password !== password) {
      throw new BusinessError(ErrorCode.BAD_PASSWORD, '密码错误');
    }
    const userinfo = _.pick(entity, [ 'userName', 'openid' ]);
    const { secret, expiresIn = '1h', issuer = 'naf' } = this.config.jwt;
    const token = await jwt.sign(userinfo, secret, { expiresIn, issuer, subject: `${userName}@${this.tenant}` });
    return { userinfo, token };
  }

  // 修改密码
  async editPwa({ userName, oldpass, newpass }) {
    assert(userName, '用户名不能为空');
    assert(oldpass, '原有密码不能为空');
    assert(newpass, '新密码不能为空');

    // 检查用户是否存在
    const entity = await this.model.findOne({ userName });
    if (!entity) throw new BusinessError(ErrorCode.DATA_NOT_EXIST, '用户不存在');
    // 校验旧密码
    if (!entity.password || entity.password !== oldpass) {
      throw new BusinessError(ErrorCode.BAD_PASSWORD, '原密码错误');
    }
    if (entity.password) {
      entity.password = newpass;
    }
    await entity.save();
  }

  // 二维码登录
  async qrcodelogin({ uuid }) {
    const { code } = this.ctx.query;
    if (code) {
      return await this.Back({ code, uuid });
    }
    // TODO: 生成回调地址
    const { wxapi, authUrl = this.ctx.path } = this.app.config;
    const host = this.ctx.header['x-forwarded-host'];
    const backUrl = encodeURI(`${this.ctx.protocol}://${host}${authUrl}`);
    const to_uri = `${wxapi.baseUrl}/api/auth?appid=${wxapi.appid}&response_type=code&redirect_uri=${backUrl}#wechat`;
    this.ctx.redirect(to_uri);
  }
  // GET 认证回调
  async Back({ code, uuid }) {
    const { weixin } = this.ctx.service;
    const res = await weixin.fetch(code);
    const openid = res.openid;
    await this.ctx.render('weixinlogin.njk', { openid, uuid });
  }
  // 检查uuid是否过期与uuid状态
  async checkQrcode({ uuid, openid }) {
    // 验证二维码
    const val = await this.app.redis.get(uuid);
    if (!val) {
      return { errcode: -1001, errmsg: '二维码已过期' };
    }
    if (val !== 'ready') {
      return { errcode: -1001, errmsg: '二维码已失效' };
    }
    // 验证用户
    const res = await this.ctx.service.admin.query({ openid });
    if (res.length <= 0) {
      return { errcode: -1001, errmsg: '微信用户不存在' };
    }
    const msg = JSON.stringify(await this.adminlogin(res[0]));
    await this.app.redis.set(uuid, msg, 'EX', 600);
    return;
  }

  // 使用二维码换取登录凭证
  async qrcodeToken(uuid) {
    assert(uuid, 'uuid不能为空');
    const val = await this.app.redis.get(uuid);
    if (!val) {
      throw new BusinessError(ErrorCode.SERVICE_FAULT, '二维码已过期');
    }
    if (val === 'ready' || val === 'consumed') {
      throw new BusinessError(ErrorCode.SERVICE_FAULT, '二维码状态无效');
    }

    // TODO: 修改二维码状态
    await this.app.redis.set(uuid, 'consumed', 'EX', 600);

    return val;
  }
}

module.exports = JwtLoginService;
